/* ###*B*###
 * ERIKA Enterprise - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2012 Evidence S.r.l.
 *
 * This file is part of ERIKA Enterprise.
 *
 * ERIKA Enterprise is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation, 
 * (with a special exception described below).
 *
 * Linking this code statically or dynamically with other modules is
 * making a combined work based on this code.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this code with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * ERIKA Enterprise is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 * ###*E*### */

#include "data_model.h"
#if defined(_MSC_VER)
	typedef __int32 int32_t;
	typedef __int16 int16_t;
	typedef __int8 int8_t;
#else
	#error "Compiler not supported"
#endif
#include <stdlib.h>
#include <string.h>
#include <assert.h>

int dm_types_size[] = {sizeof(int8_t), sizeof(int16_t), sizeof(int32_t), sizeof(float),
				   sizeof(double)};

void dm_init(struct dm_elem* data)
{
	data->data_ = 0;
	data->size_ = 0;
	data->description_.types_ = 0;
	data->description_.size_ = 0;
}

int dm_get_value(const struct dm_elem* data, int index, unsigned char* type,
			  void** value)
{
	int offset = 0, i, value_size;
	if (index < 0 || index >= data->description_.size_)
	{
		return 0;
	}
	for(i = 0; i < index; ++i)
	{
		assert(data->description_.types_[i] <
			   (sizeof(dm_types_size)/sizeof(int)));
		if (data->description_.types_[i] <
				(sizeof(dm_types_size)/sizeof(int)))
		{
			offset = offset + dm_types_size[data->description_.types_[i]];
		}
	}
	*type = data->description_.types_[index];
	assert(*type < (sizeof(dm_types_size)/sizeof(int)));
	if (*type < (sizeof(dm_types_size)/sizeof(int)))
	{
		value_size = dm_types_size[*type];
	}
	else
	{
		return 0;
	}
	*value = malloc(value_size);
	assert(offset+value_size <= data->size_);
	if (offset+value_size <= data->size_)
	{
		memcpy(*value, (const char*)data->data_+offset, value_size);
	}
	return value_size;
}

void dm_set_value(struct dm_elem *data, int index, const void *value, int size)
{
	int offset = 0, i, value_size;
	unsigned char type;
	if (index < 0 || index >= data->description_.size_)
	{
		return;
	}
	for(i = 0; i < index; ++i)
	{
		assert(data->description_.types_[i] <
			   (sizeof(dm_types_size)/sizeof(int)));
		if (data->description_.types_[i] <
				(sizeof(dm_types_size)/sizeof(int)))
		{
			offset = offset + dm_types_size[data->description_.types_[i]];
		}
	}
	type = data->description_.types_[index];
	assert(type < (sizeof(dm_types_size)/sizeof(int)));
	if (type < (sizeof(dm_types_size)/sizeof(int)))
	{
		value_size = dm_types_size[type];
	}
	assert(value_size == size && offset < data->size_);
	if (value_size == size && offset+size <= data->size_)
	{
		memcpy((char*)data->data_+offset, value, size);
	}
}

void dm_create_elem_descr(struct dm_elem_descr* descr, unsigned char* types,
						   int size)
{
	if (types)
	{
		descr->types_ = malloc(size);
		descr->size_ = size;
		memcpy(descr->types_, types, size);
	}
}

void dm_erase_elem_descr(struct dm_elem_descr* descr)
{
	free(descr->types_);
	descr->types_ = 0;
	descr->size_ = 0;
}

void dm_create_elem(struct dm_elem *data, unsigned char* types,
					int size)
{
	int i, data_size = 0;
	unsigned char type;
	dm_create_elem_descr(&data->description_, types, size);
	for(i = 0; i < size; ++i)
	{
		type = data->description_.types_[i];
		assert(type < (sizeof(dm_types_size)/sizeof(int)));
		if (type < (sizeof(dm_types_size)/sizeof(int)))
		{
			data_size += dm_types_size[type];
		}
	}
	if (data_size > 0)
	{
		data->data_ = malloc(data_size);
		data->size_ = data_size;
	}
	else
	{
		data->data_ = 0;
		data->size_ = 0;
	}
}

void dm_erase_elem(struct dm_elem *data)
{
	dm_erase_elem_descr(&data->description_);
	free(data->data_);
	data->data_ = 0;
	data->size_ = 0;
}

void dm_create_types(unsigned char **types_out, const char *types, int size)
{
	int i;
	int j=0;
	for(i=0; i<size; ++i)
	{
		if (types[i] == 'c')
		{
			*types_out = realloc(*types_out, j+1);
			(*types_out)[j] = INT8_TYPE;
			++j;
		}
		else if (types[i] == 's')
		{
			*types_out = realloc(*types_out, j+1);
			(*types_out)[j] = INT16_TYPE;
			++j;
		}
		else if (types[i] == 'i')
		{
			*types_out = realloc(*types_out, j+1);
			(*types_out)[j] = INT32_TYPE;
			++j;
		}
		else if (types[i] == 'f')
		{
			*types_out = realloc(*types_out, j+1);
			(*types_out)[j] = FLOAT_TYPE;
			++j;
		}
		else if (types[i] == 'd')
		{
			*types_out = realloc(*types_out, j+1);
			(*types_out)[j] = DOUBLE_TYPE;
			++j;
		}
	}
}

void dm_erase_types(unsigned char **types_out)
{
	free(*types_out);
	*types_out = 0;
}
